/*  MapEditor.cpp created Nov 23, 2010
 *
 *  Copyright (C) 2010 Alexey Abel
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *
 */

#include "MapEditor.h"
#include <QGraphicsRectItem>
#include <QPen>
#include <QGraphicsLineItem>
#include <QGraphicsSceneMouseEvent>//TODO: ?

MapEditor::MapEditor(Board& parentBoard) : parentBoard_(parentBoard), lastChangedCell_(CellCoordinate(0,0))

{
	// Create 2 dimensional list for matrix representation, editing
	MapEditorCell* cell;
	for (uint column=0; column<parentBoard_.columnCount(); ++column)
	{
		editorMatrix_.append(QList <MapEditorCell*> ());
		for (uint row=0; row<parentBoard_.rowCount(); ++row)
		{
			cell = new MapEditorCell(parentBoard_.cell(column,row).isWalkable(),parentBoard_.cell(column,row).isBuildable());
			cell->setPos(column*10,row*10);
			this->addToGroup(cell);
			editorMatrix_[column].append(cell); //TODO: ASSERT that QT doesnt delete objects, this could be empty pointer, maybe use shared.pointer

		}
	}

	setHandlesChildEvents(true);
	setCacheMode(DeviceCoordinateCache);
	setFiltersChildEvents(true);

	// Create visual grid
	// vertical lines
	QGraphicsLineItem* line;
	for (uint column=1; column<parentBoard_.columnCount(); column++) //here one could use local editorMatrix_ dimensions and not the value from board, although they both must be the same
	{
		line = new QGraphicsLineItem(qreal(column*10), 0.0, qreal(column*10), qreal(td::boardHeight()));
		line->setZValue(100.0);
		this->addToGroup(line);
	}
	// horizontal lines
	for (uint row=1; row<parentBoard_.rowCount(); row++)
	{
		line = new QGraphicsLineItem(0.0, qreal(row*10), qreal(td::boardWidth()), qreal(row*10));
		line->setZValue(100.0);
		this->addToGroup(line);
	}

	//lastChangedCell_ = CellCoordinate(0,0); // TODO: find a better solution
}



MapEditor::~MapEditor()
{
	// TODO Auto-generated destructor stub
}



MapEditorCell& MapEditor::cell(uint column, uint row)
{
	//TODO: ASSERT legal index
	return *editorMatrix_[column][row];
}



MapEditorCell& MapEditor::cell(CellCoordinate cellcoordinate)
{
	//TODO: ASSERT legal index
	return *editorMatrix_[cellcoordinate.column()][cellcoordinate.row()];
}



void MapEditor::mousePressEvent ( QGraphicsSceneMouseEvent * event )
{
	//TODO: optimize the multiple CellCoordinate(event->pos()) calls here
	if (event->buttons() == Qt::LeftButton)
	{
		//qDebug() << "press at (" << CellCoordinate(event->pos()).column() << "," << CellCoordinate(event->pos()).row() << ")";
		if (parentBoard_.cell(CellCoordinate(event->pos())).isWalkable())
		{
			currentMouseAction_ = walkableOff;
			modifyCell(CellCoordinate(event->pos()));
		}
		else
		{
			currentMouseAction_ = walkableOn;
			modifyCell(CellCoordinate(event->pos()));
		}
	}
	else if (event->buttons() == Qt::RightButton)
	{
		if (parentBoard_.cell(CellCoordinate(event->pos())).isBuildable())
		{
			currentMouseAction_ = buildableOff;
			modifyCell(CellCoordinate(event->pos()));
		}
		else
		{
			currentMouseAction_ = buildableOn;
			modifyCell(CellCoordinate(event->pos()));
		}
	}
}



void MapEditor::mouseReleaseEvent ( QGraphicsSceneMouseEvent * event )
{
	//qDebug() << "release at (" << CellCoordinate(event->pos()).column() << "," << CellCoordinate(event->pos()).row() << ")";

}



void MapEditor::mouseMoveEvent ( QGraphicsSceneMouseEvent * event )
{
	if (boundingRect().adjusted(0,0,-1,-1).contains(event->pos()))//TODO: Why is boundingRect bigger than the item?
	{
		//TODO: optimize the multiple CellCoordinate(event->pos()) calls here
		//qDebug() << "move in (" << CellCoordinate(event->pos()).column() << "," << CellCoordinate(event->pos()).row() << ")";
		if (CellCoordinate(event->pos()) != lastChangedCell_) {
			lastChangedCell_ = CellCoordinate(event->pos());
			modifyCell(CellCoordinate(event->pos()));
		}
	}
}

void MapEditor::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event )
{
	if (event->buttons() == Qt::RightButton)
	{
		CellCoordinate oldGoal = parentBoard_.goal();
		parentBoard_.setGoal(CellCoordinate(event->pos()));
		cell(oldGoal).setGoal(false);
		cell(CellCoordinate(event->pos())).setGoal(true);
		// needed here because one right click is catched to and toggles buildable
		// so we change it back. TODO: find out how to only catch a double click
		cell(CellCoordinate(event->pos())).toggleBuildable();

	}
}



//TODO: why doesnt pass by reference work?
void MapEditor::modifyCell (CellCoordinate /*const*/   cell) //TODO: re-const this and make const constuctor for CellCoordinate
{
	qDebug()<< "path_.push_back(CellCoordinate("<< cell.column() <<","<< cell.row() <<"));";
	//TODO: solve this with function pointers?
	//TODO: combine visualisation change with matrix change. maybe call matrix change from visu change function
	if (currentMouseAction_ == walkableOn)
	{
		parentBoard_.cell(CellCoordinate(cell)).setWalkable(true);
		this->cell(cell).setWalkable(true);
	}
	else if (currentMouseAction_ == walkableOff)
	{
		parentBoard_.cell(CellCoordinate(cell)).setWalkable(false);
		this->cell(cell).setWalkable(false);
	}
	else if (currentMouseAction_ == buildableOn)
	{
		parentBoard_.cell(CellCoordinate(cell)).setBuildable(true);
		this->cell(cell).setBuildable(true);
	}
	else if (currentMouseAction_ == buildableOff)
	{
		parentBoard_.cell(CellCoordinate(cell)).setBuildable(false);
		this->cell(cell).setBuildable(false);

	}
}

//const not used yet. delete if never user
//const Cell& MapEditor::cellByPoint (const QPointF& point) const
//{
//    uint x = (point.x() - point.x() % 10)/10;
//    uint y = (point.y() - point.y() % 10)/10;
//    return cells_.at(x).at(y);
//}


////////////////////MapEditorCell////////////////////////////////////

MapEditorCell::MapEditorCell(bool walkable, bool buildable) :
		QGraphicsRectItem(QRectF(0, 0, td::cellSize(), td::cellSize())), walkable_(walkable), buildable_(buildable),isGoal_(false)
{
	updateColor();
	setZValue(1);
}

void MapEditorCell::toggleWalkable()
{
	walkable_ = !walkable_;
	updateColor();
	update(this->boundingRect());
}
void MapEditorCell::toggleBuildable()
{
	buildable_ = !buildable_;
	updateColor();
	update(this->boundingRect());
}



void MapEditorCell::setWalkable(bool walkable) {
	if (walkable_ != walkable)
	{
		walkable_ = walkable;
		updateColor();
		update(this->boundingRect());
	}
}



void MapEditorCell::setBuildable(bool buildable) {
	if (buildable_ != buildable)
	{
		buildable_ = buildable;
		updateColor();
		update(this->boundingRect());
	}
}

void MapEditorCell::setGoal(bool isGoal) {
	isGoal_ = isGoal;
	updateColor();
}



void MapEditorCell::updateColor()
{
	//TODO: make the colors here enums in td:: ?
	if (isGoal_)
	{
		setBrush(QBrush(QColor(255,0,0,255)));
		setPen(Qt::NoPen);
		return;
	}
	if (walkable_)
		if (buildable_) {

			setBrush(QBrush(QColor(222,255,210,255)));
			setPen(Qt::NoPen);
		}
		else {
			setBrush(QBrush(QColor(255,255,255,255)));
			setPen(Qt::NoPen);
		}
	else
		if (buildable_) {
			setBrush(QBrush(QColor(55,136,35,255)));

			setPen(Qt::NoPen);
		}
		else {
			setBrush(QBrush(QColor(50,50,50,255)));
			setPen(Qt::NoPen);
		}

}

MapEditorCell::~MapEditorCell()
{
	// TODO Auto-generated destructor stub
}
